import { defineStore } from 'pinia';
import type { RouteLocationNormalizedLoaded } from 'vue-router';
import cloneDeep from 'lodash-es/cloneDeep';
import type { Layout, MenuList, ISetting, ITabs, IMenubar, ITabsList } from '@/types/store/layout';
import { IMenubarStatus } from '@/types/store/layout';
import { generatorDynamicRouter } from '@/router/asyncRouter';
import router, { allowRouter } from '@/router/index';
import { getLocal } from '@/utils/tools';
import { getRouterList } from '@/api/login';

const setting = getLocal<ISetting>('setting');

export const useLayoutStore = defineStore({
  id: 'layout',
  state: (): Layout => ({
    tabs: {
      tabsList: [],
      cachedViews: [],
    },
    setting: {
      theme: setting.theme !== undefined ? setting.theme : 0,
      color: {
        primary: setting.color && setting.color.primary ? setting.color.primary : '#3691FF',
        headerBg: setting.color && setting.color.headerBg ? setting.color.headerBg : '#3691FF',
      },
      showTabs: setting.showTabs !== undefined ? setting.showTabs : true,
      mode: setting.mode || 'horizontal',
    },
    menubar: {
      status: document.body.offsetWidth < 768 ? IMenubarStatus.PHN : IMenubarStatus.PCE,
      menuList: [],
      subMenuList: [],
      isPhone: document.body.offsetWidth < 768,
    },
  }),
  getters: {
    getSetting(): ISetting {
      return this.setting;
    },
    getTabs(): ITabs {
      return this.tabs;
    },
    getMenubar(): IMenubar {
      return this.menubar;
    },
  },
  actions: {
    setRoutes(data: Array<MenuList>): void {
      this.menubar.menuList = data;
    },
    setSubMenuList(data: Array<MenuList>): void {
      this.menubar.subMenuList = data;
    },
    // 修改主题
    changeTheme(num?: number): void {
      let tempNum = num;
      if (tempNum === this.setting.theme) return;
      if (typeof tempNum !== 'number') tempNum = this.setting.theme;
      this.setting.theme = tempNum;
      localStorage.setItem('setting', JSON.stringify(this.setting));
    },
    // 修改主题色
    changeThemeColor(color: string): void {
      this.setting.color.primary = color;
      localStorage.setItem('setting', JSON.stringify(this.setting));
    },
    // 修改头部标题色
    changeHeaderBgColor(color: string): void {
      this.setting.color.headerBg = color;
      localStorage.setItem('setting', JSON.stringify(this.setting));
    },
    concatAllowRoutes(): void {
      const curMenuList = cloneDeep(this.menubar.menuList);
      allowRouter[0].children!.push(...curMenuList);
      this.menubar.menuList = allowRouter;
    },
    async GenerateRoutes(): Promise<void> {
      const res = await getRouterList();
      const { data } = res;
      generatorDynamicRouter(data);
    },
    // 修改导航模式
    async changeMenuBarMode(mode: 'horizontal' | 'vertical' | 'both') {
      this.menubar.menuList = [];
      this.menubar.subMenuList = [];
      this.concatAllowRoutes();
      this.setting.mode = mode;
      localStorage.setItem('setting', JSON.stringify(this.setting));
    },
    changeCollapsed(): void {
      const isPCN =
        this.menubar.status === IMenubarStatus.PCN ? IMenubarStatus.PCE : IMenubarStatus.PCN;
      this.menubar.status = this.menubar.isPhone
        ? this.menubar.status === IMenubarStatus.PHN
          ? IMenubarStatus.PHE
          : IMenubarStatus.PHN
        : isPCN;
    },
    removeAllTabNav(): void {
      this.tabs.tabsList.splice(0);
      this.tabs.cachedViews.splice(0);
      router.push({ path: '/redirect/' });
    },
    // 添加缓存页面
    addCachedViews(obj: { name: string; noCache: boolean }): void {
      if (!this.setting.showTabs) return; // 判断是否开启多标签页
      if (obj.noCache || this.tabs.cachedViews.includes(obj.name)) return;
      this.tabs.cachedViews.push(obj.name);
    },
    removeTabNav(obj: { tabsList: ITabsList; cPath: string }): void {
      const index = this.tabs.tabsList.findIndex((v) => v.path === obj.tabsList.path);
      if (this.tabs.tabsList[index].path === obj.cPath) {
        this.tabs.tabsList.splice(index, 1);
        const i = index === this.tabs.tabsList.length ? index - 1 : index;
        this.tabs.tabsList[i].isActive = true;
        this.removeCachedViews({
          name: obj.tabsList.name,
          index,
        });
        router.push({
          path: this.tabs.tabsList[i].path,
        });
      } else {
        this.tabs.tabsList.splice(index, 1);
        this.removeCachedViews({
          name: obj.tabsList.name,
          index,
        });
      }
    },
    // 删除缓存页面
    removeCachedViews(obj: { name: string; index: number }): void {
      // 判断标签页是否还有该页面
      if (this.tabs.tabsList.map((v) => v.name).includes(obj.name)) return;
      this.tabs.cachedViews.splice(obj.index, 1);
    },
    // 删除其他标签页
    removeOtherTabNav(tabsList: { path: string }): void {
      const index = this.tabs.tabsList.findIndex((v) => v.path === tabsList.path);
      this.tabs.tabsList.splice(index + 1);
      this.tabs.tabsList.splice(0, index);
      this.tabs.cachedViews.splice(index + 1);
      this.tabs.cachedViews.splice(0, index);
      router.push({ path: tabsList.path });
    },
    // 切换导航，记录打开的导航
    changeTabNavList(cRouter: RouteLocationNormalizedLoaded): void {
      if (!this.setting.showTabs) return; // 判断是否开启多标签页
      if (cRouter.meta.hidden && !cRouter.meta.activeMenu) return; // 隐藏的菜单如果不是子菜单则不添加到标签
      // eslint-disable-next-line prefer-regex-literals
      if (new RegExp('^/redirect').test(cRouter.path)) return;
      const index = this.tabs.tabsList.findIndex((v) => v.path === cRouter.path);
      this.tabs.tabsList.forEach((v) => {
        v.isActive = false;
      });
      // 判断页面是否打开过
      if (index !== -1) {
        this.tabs.tabsList[index].isActive = true;
        return;
      }
      const tabsList: ITabsList = {
        name: cRouter.name as string,
        title: cRouter.meta.title as string,
        path: cRouter.path,
        isActive: true,
      };
      this.tabs.tabsList.push(tabsList);
    },
    changeTabsSetting(showTabs: boolean): void {
      this.setting.showTabs = showTabs;
      localStorage.setItem('setting', JSON.stringify(this.setting));
      if (showTabs) {
        const index = this.tabs.tabsList.findIndex(
          (v) => v.path === router.currentRoute.value.path,
        );
        if (index !== -1) {
          this.tabs.tabsList.forEach((v) => {
            v.isActive = false;
          });
          this.tabs.tabsList[index].isActive = true;
        } else {
          this.changeTabNavList(router.currentRoute.value);
        }
      }
    },
  },
});
