import type { AxiosResponse } from 'axios';
import axios from 'axios';
import { ElNotification, ElMessageBox, ElMessage } from 'element-plus';
import { Buffer } from 'buffer';
import { useSystemStore } from '@/store/modules/system';
import { refreshToken } from '@/api/login';
// 标记是否正在刷新 token
let isRefreshing = false;
// 存储待重发请求的数组
let requests: any[] = [];
// 创建 axios 实例
const request = axios.create({
  // API 请求的默认前缀
  baseURL: import.meta.env.VITE_APP_BASE_URL as string,
  timeout: 60000, // 请求超时时间
});
// 异常拦截处理器
const errorHandler = (error: any) => {
  // loading.close()

  if (!error.response) {
    ElNotification({
      title: '请求失败',
      message: '请求超时请检查网络链接',
      type: 'error',
    });
  } else {
    const { data } = error.response;
    ElNotification({
      title: '请求失败',
      message: data.message,
      type: 'error',
    });
  }
  return Promise.reject(error);
};

request.interceptors.request.use((config) => {
  // mock接口去除baseURL
  if (config.url?.startsWith('/api')) {
    config.baseURL = '';
  }
  if (config.url === '/system/login/captcha') {
    // 验证码接口不用带token
    return config;
  } else if (config.url === '/auth/oauth/token') {
    // 登录接口，指定请求头
    const str = 'lin:123123';
    const str64 = Buffer.from(str).toString('base64');
    config.headers!.Authorization = `Basic ${str64}`;
  } else {
    const token = localStorage.getItem('TOKEN');
    if (token) {
      config.headers!.Authorization = `Bearer ${JSON.parse(token).access_token}`;
    } else if (useSystemStore().getTempToken) {
      config.headers!.Authorization = `Bearer ${useSystemStore().getTempToken.access_token}`;
    }
  }
  return config;
}, errorHandler);

// response interceptor
request.interceptors.response.use(
  (response: AxiosResponse) => {
    const { config, data, headers } = response;
    // mock接口直接返回
    if (config.url?.startsWith('/api')) {
      return data;
    }
    // 返回文件流数据提交文件名，返回名称和数据
    if (!data.code && data.type !== 'application/json') {
      let fileName = '';
      if (headers['content-disposition']) {
        const arr = headers['content-disposition'].split(';');
        // eslint-disable-next-line no-restricted-syntax
        for (const item of arr) {
          if (item.search('filename') !== -1) {
            const arr1 = item.split('=');
            fileName = decodeURIComponent(arr1[arr1.length - 1]);
            break;
          }
        }
      }
      return {
        data: {
          data: data,
          fileName: fileName,
        },
      };
    }
    if (!data.code && data.type === 'application/json') {
      const reader = new FileReader();
      reader.readAsText(data, 'utf-8');
      reader.onload = () => {
        const data = JSON.parse(reader.result as string);
        ElMessage.error(data.message);
      };
      return;
    }
    if (response?.status === 200) {
      const { code } = data;
      // 0000：操作成功
      // 0001：操作成功（前端不弹窗）
      // 2016：提示页面修改密码，允许通过
      if (['0000', '0001', '2016'].includes(code)) {
        if (config.method === 'post' && code !== '0001') {
          if (document.getElementsByClassName('el-message').length === 0) {
            ElMessage.success(data.message);
          }
        }
        return data;
      } else if (['2019'].includes(code)) {
        // 2019：密码到期,请修改密码！
        ElMessage.warning(data.message);
        return data;
      } else {
        // 重新登录
        if (code === '2008') {
          ElMessageBox.confirm(data.message, {
            confirmButtonText: 'OK',
            type: 'warning',
            closeOnClickModal: false,
            closeOnPressEscape: false,
            showCancelButton: false,
            showClose: false,
          }).then(() => {
            useSystemStore().logout();
          });
          return;
        }

        // 2009: 用户已经登陆，不可以重复登陆！
        if (!['2009'].includes(code)) {
          ElMessage.error(data.message);
        }

        // 防止外层等不到回调
        return Promise.reject(response);
      }
    } else return Promise.reject(response);
  },
  (error) => {
    if (error.response) {
      // if (
      //   error.response.status === 401 &&
      //   error.response.data.code === '2007' &&
      //   !error.config.url.includes('/auth/oauth/token')
      // ) {
      //   useSystemStore().logout();
      // }

      // ElMessage.error(error.response?.data?.message || '请求失败');
      // return Promise.reject(error.response);
      if (
        error.response.status === 401 &&
        error.response.data.code === '2007' &&
        !error.config.url.includes('/auth/oauth/token')
      ) {
        const { config } = error;
        if (!isRefreshing) {
          isRefreshing = true;
          return refreshToken()
            .then((res) => {
              const { access_token: accessToken } = res.data;
              useSystemStore().saveToken(res.data);
              config.headers.Authorization = `Bearer ${accessToken}`;
              // token 刷新后将数组的方法重新执行
              requests.forEach((cb) => cb(accessToken));
              requests = []; // 重新请求完清空
              return request(config);
            })
            .catch((err) => {
              console.error('抱歉，您的登录状态已失效，请重新登录！');
              return Promise.reject(err);
            })
            .finally(() => {
              isRefreshing = false;
            });
        } else {
          // 返回未执行 resolve 的 Promise
          return new Promise((resolve) => {
            // 用函数形式将 resolve 存入，等待刷新后再执行
            requests.push((token: string) => {
              config.headers.Authorization = `Bearer ${token}`;
              resolve(request(config));
            });
          });
        }
      } else {
        ElMessage.error(error.response?.data?.message || '请求失败');
        return Promise.reject(error.response);
      }
    }
  },
);

export default request;
