import router from '@/router/index';
import type { LocalStore } from '@/types/tools';
import type { MenuList } from '@/types/store/layout';
import { getDictInfo as getDictInfoApi } from '@/api/systemManagement/dictionaryManagement';
import type { IObject } from '@/types/global';
import { useSystemStore } from '@/store/modules/system';

export type Item = {
  label: string;
  value: string;
  children?: Item[];
};
export type AllData = { [key: string]: Item[] };

/**
 * 睡眠函数
 * @param time
 */
export async function sleep(time: number): Promise<void> {
  await new Promise((resolve) => {
    setTimeout(() => resolve, time);
  });
}

/**
 * 金额格式化
 * @param num 金额
 * @param symbol 金额前面修饰符号，如$,￥
 */
export function format(num: number | string, symbol = '￥'): string {
  if (Number.isNaN(Number(num))) return `${symbol}0.00`;
  return (
    symbol +
    Number(num)
      .toFixed(2)
      .replace(/(\d)(?=(\d{3})+\.)/g, '$1,')
  );
}

/**
 * 取消金额格式化
 * @param str 金额
 */
export function unformat(str: string): number | string {
  const s = str.substr(1).replace(/,/g, '');
  return Number.isNaN(Number(s)) || Number(s) === 0 ? '' : Number(s);
}

/**
 * 表格合计行
 * @param str 金额
 */
export function tableSummaries(param: { columns: any; data: any }): Array<string | number> {
  const { columns, data } = param;
  const sums: Array<string | number> = [];
  columns.forEach((column: { property: string | number }, index: number) => {
    if (index === 0) {
      sums[index] = '合计';
      return;
    }
    const values = data.map((item: { [x: string]: any }) => Number(item[column.property]));
    if (!values.every((value: number) => Number.isNaN(value))) {
      sums[index] = values.reduce((prev: number, curr: number) => {
        const value = Number(curr);
        if (!Number.isNaN(value)) {
          return prev + curr;
        } else {
          return prev;
        }
      }, 0);
      const sum = sums[index];
      if (typeof sum === 'number') {
        sums[index] = format(sum.toFixed(2));
      }
    } else {
      sums[index] = 'N/A';
    }
  });

  return sums;
}

export function isInput(el: HTMLElement): boolean {
  return el.nodeName.toLocaleLowerCase() === 'input';
}
export function isTextarea(el: HTMLElement): boolean {
  return el.nodeName.toLocaleLowerCase() === 'textarea';
}

/**
 * localStorage设置有效期
 * @param name localStorage设置名称
 * @param data 数据对象
 * @param pExpires 有效期(默认100年)
 */
export function setLocal(
  name: string,
  data: IObject<any>,
  pExpires = 1000 * 60 * 60 * 24 * 365 * 100,
): void {
  const d = data as LocalStore;
  d.startTime = Date.now();
  d.expires = pExpires;
  localStorage.setItem(name, JSON.stringify(data));
}

/**
 * 获取localStorage对象并转成对应的类型
 * @param name localStorage设置名称
 */
export function getLocal<T>(name: string): T {
  const l = localStorage.getItem(name);
  const local = JSON.parse(l !== null ? l : '{}') as unknown as T;
  return local;
}

/**
 * 字符串首字母大写
 * @param str
 * @returns
 */
export function firstUpperCase(str: string): string {
  return str.replace(/^\S/, (s) => s.toUpperCase());
}

/**
 * 两次编码url
 * @param url
 * @returns
 */
export function decode(url: string): string {
  return decodeURIComponent(decodeURIComponent(url));
}

/**
 * 两次解码url
 * @param url
 * @returns
 */
export function encode(url: string): string {
  return encodeURIComponent(encodeURIComponent(url));
}

// date对象格式化日期为yyyy-MM-dd
export function handleFormatDate(date: any): string {
  if (!date) {
    return '';
  }
  const Y = date.getFullYear();
  const M = date.getMonth() + 1;
  const D = date.getDate();
  const hour = date.getHours();
  const minute = date.getMinutes();
  const second = date.getSeconds();
  const times =
    Y +
    (M < 10 ? '-0' : '-') +
    M +
    (D < 10 ? '-0' : '-') +
    D +
    (hour < 10 ? ' 0' : ' ') +
    hour +
    (minute < 10 ? ':0' : ':') +
    minute +
    (second < 10 ? ':0' : ':') +
    second;
  return times;
}

// 动态创建<a>标签下载文件
export function createToDownloadFile(data: Blob, fileName: string): void {
  let domHref: HTMLAnchorElement | null = document.createElement('a');
  const blobUrl = window.URL.createObjectURL(data);
  domHref.href = blobUrl;
  domHref.download = fileName;
  domHref.click();
  domHref = null;
  window.URL.revokeObjectURL(blobUrl);
}

/**
 * 提取下拉数据
 */
export function getValue(data: any[]): Item[] {
  return data.map((i) => {
    const obj: Item = {
      label: i.dictValue,
      value: i.dictKey,
    };
    if (i.children?.length) {
      obj.children = getValue(i.children);
    }
    return obj;
  });
}

/**
 * @description 获取所有下拉值对象
 * @param {Array} codeList
 */
export async function getDictInfo(codeList: Record<'code' | 'prop', string>[]): Promise<AllData> {
  const codes = codeList.map((i) => i.code);

  const selectAllData: any = {};
  await getDictInfoApi(codes.toString()).then((res: any) => {
    codeList.forEach((item) => {
      selectAllData[item.prop] = getValue(res.data[item.code]);
    });
  });

  return selectAllData;
}

/**
 * 函数防抖 某个时间内只执行最后一次
 * @param func 回调函数
 * @param time 间隔时间
 */
export function debounce(func: () => void, time = 500): () => void {
  let timer: any = null;
  return function (...arg) {
    if (timer) {
      clearTimeout(timer);
    }
    timer = setTimeout(() => {
      // @ts-ignore
      func.apply(this, arg);
    }, time);
  };
}

/**
 * 函数节流 某个时间内只执行第一次
 * @param func 回调函数
 * @param time 间隔时间
 */
export function throttle(func: () => void, time = 500): () => void {
  let timer: any = null;
  return function (...arg) {
    if (timer) {
      return;
    }
    timer = setTimeout(() => {
      clearTimeout(timer);
      timer = null;
      // @ts-ignore
      func.apply(this, arg);
    }, time);
  };
}

/**
 * 前三位后四位中间加星号
 * @param {string} value 要加星号的值
 * @param {number} num 星号的个数
 */
export function asterisk(value: string, num = 4): string {
  if (value.length > 7) {
    const ast = Array.from({ length: num }, () => '*').join('');
    const start = value.substr(0, 4);
    const end = value.substr(value.length - 3);
    return start + ast + end;
  } else return value;
}

// 拍平路由数组
export function flatRouterArray(routerArr: MenuList[]): MenuList[] {
  return routerArr.reduce(
    (prev, cur) =>
      prev.concat(
        Array.isArray(cur.children) && cur.children.length > 0
          ? flatRouterArray(cur.children)
          : (cur as any),
      ),
    [],
  );
}

// 获取权限标识及资源操作权限
export const fetchPermissionOperate = (value: string): any[] => {
  const { operate } = router.currentRoute.value.meta;
  const currentRoutePermission = (router.currentRoute.value.meta.permission || []) as any[];
  const roles: Array<string> = currentRoutePermission;
  const hasPermission = roles.includes(value);
  return [hasPermission, operate];
};

// 判断是否有权限进行当前操作
export const checkPermission = (value: string): boolean => {
  const currentRoutePermission = (router.currentRoute.value.meta.permission || []) as any[];
  const roles: Array<string> = currentRoutePermission;
  const isShow = roles.includes(value);
  return isShow;
};

/**
 * 路由list结构转tree并组织成 MenuList的结构
 * @param data list原始数据
 * @param pid 最外层pid
 */
export function listToTree(data: Array<any>, pid: string | number = 1): Array<MenuList> {
  const d: Array<MenuList> = [];
  data.forEach((val) => {
    if (val.pid === pid) {
      const { name, path, icons, isCache, hidden, cmpPath, permission, type, operate } = val;
      const menuItem = {
        name: path,
        path,
        component: cmpPath,
        type,
        meta: { title: name, icon: icons, hidden, isCache, permission, operate },
      };

      const list = listToTree(data, val.id);
      // type: -1 表示是菜单， 下一级就是按钮
      if (Number(type) === -1) {
        const permissions = list.map((item) => item.meta.permission);
        menuItem.meta.permission = permissions;
      }
      const obj: MenuList = { ...menuItem };
      // type === 0 是目录 而且存在子节点
      if (Number(type) === 0 && list.length !== 0) {
        obj.children = list;
      }
      // type === 2 是虚拟目录需要计入路由
      if (Number(type) === 2 && list.length !== 0) {
        obj.children = list;
      }
      d.push(obj);
    }
  });
  return d;
}

/**
 * @description 校验非零正整数
 * @param str 输入值
 */
export const handleCheckPositiveInteger = (str: string) => /^\+?[1-9][0-9]*$/.test(str);

/**
 * @description 校验邮箱
 * @param str 输入值
 */
export const handleCheckIsEmail = (str: string) =>
  /^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$/.test(str);

/**
 * @description 校验ip
 * @param str 输入值
 */
export const handleCheckIsIP = (str: string) =>
  /^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])$/.test(
    str,
  );

// 通过 sendBeacon 在关闭浏览器时调用退出登录接口
export const logoutBeforeCloseTheBrowser = () => {
  const token = localStorage.getItem('TOKEN');
  const access_token = token
    ? JSON.parse(token).access_token
    : useSystemStore().getTempToken.access_token;
  if (access_token) {
    window.navigator.sendBeacon(
      `${import.meta.env.VITE_APP_BASE_URL}/system/user/userLogoutByToken?token=${access_token}`,
    );
  }
};
