import type { ComputedRef, Ref } from 'vue';
import { computed, reactive, watch, ref } from 'vue';
import type { FormOption } from '@/types/tools';
import type { MenuTreeDataItem } from '@/types/api/resourceManagement';

type MyFormOption = Omit<FormOption, 'disabled'> & {
  hidden?: boolean | ComputedRef<boolean>;
  disabled?: boolean | ComputedRef<boolean>;
};
export default function useOptions(
  currentData: Ref<MenuTreeDataItem | null>,
  formType: 'add' | 'edit',
): any {
  const oldType = ref<string | undefined>('');
  // 判断显示表单项
  // 类型为目录、虚拟目录、外部资源不显示权限标识
  const isPermissionHidden = computed(() =>
    ['0', '2', '3'].includes(currentData.value ? currentData.value.type : ''),
  );
  // 类型为目录、按钮、虚拟目录不显示前端url
  const isPathAndCmpPathHidden = computed(() =>
    ['0', '1', '2'].includes(currentData.value ? currentData.value.type : ''),
  );

  // 类型为目录、按钮、虚拟目录、外部资源不显示组件路径
  const isCmpPathHidden = computed(() =>
    ['0', '1', '2', '3'].includes(currentData.value ? currentData.value.type : ''),
  );

  // 按钮不显示图标
  const isIconsHidden = computed(() =>
    ['1', '2'].includes(currentData.value ? currentData.value.type : ''),
  );

  // 判断是否可编辑
  const isTypeDisabled = computed(
    () => formType === 'edit' || (currentData.value?.type === '1' && oldType.value !== '2'),
  );

  // 业务资源可编辑内容
  const isDisabled = computed(() => formType === 'edit' && currentData.value?.classify === 1);

  const isSystemDisabled = computed(() => currentData.value?.classify === 1);

  const placeholder = computed(() => {
    if (currentData.value?.type === '3') {
      return '请输入外部资源完整地址，如https://www.baidu.com/';
    } else if (currentData.value?.type === '-1') {
      return '如菜单页完整地址为http://127.0.0.1/test/#/group/menu,则此处填：/group/menu';
    } else {
      return null;
    }
  });

  const typeOptions = computed(() => {
    if (oldType.value === '2') {
      return [
        { label: '菜单', value: '-1' },
        { label: '虚拟目录', value: '2' },
        { label: '按钮', value: '1' },
      ];
    } else {
      return [
        { label: '菜单', value: '-1' },
        { label: '虚拟目录', value: '2' },
        { label: '外部资源', value: '3' },
        { label: '菜单分组', value: '0' },
        { label: '按钮', value: '1', disabled: true },
      ];
    }
  });

  // 菜单类型更改事件
  const typeChangeEvent = computed(() => {
    if (formType === 'edit') {
      return () => {};
    } else {
      return (val) => {
        currentData.value!.type = val;
      };
    }
  });

  watch(
    () => currentData.value,
    (newV) => {
      oldType.value = newV?.type;
    },
  );

  const MyOptionList: MyFormOption[] = reactive([
    {
      label: '资源类型',
      key: 'classify',
      type: 'select',
      span: 20,
      labelWidth: '120px',
      disabled: true,
      options: [
        { label: '系统资源', value: 1 },
        { label: '业务资源', value: 0 },
      ],
    },
    {
      label: '父菜单名称',
      key: 'pidName',
      type: 'input',
      options: [],
      span: 20,
      labelWidth: '120px',
      disabled: true,
    },
    {
      label: '菜单类型',
      key: 'type',
      type: 'select',
      span: 20,
      tip: true,
      tipText:
        '【菜单分组】：在导航栏对菜单进行分组显示<br />【菜单】：导航栏的菜单按钮<br />【虚拟目录】：虚拟分组，不显示在页面上<br />【按钮/接口】：页面上的具体操作按钮、无界面接口等<br />【外部资源】：第三方页面',
      labelWidth: '120px',
      disabled: isTypeDisabled,
      changeEvent: typeChangeEvent,
      options: typeOptions,
    },
    {
      label: '权限标识',
      key: 'permission',
      type: 'input',
      options: [],
      span: 20,
      labelWidth: '120px',
      hidden: isPermissionHidden,
      disabled: isDisabled,
    },
    {
      label: '名称',
      key: 'name',
      type: 'input',
      span: 20,
      labelWidth: '120px',
      disabled: isDisabled,
    },
    {
      label: '描述',
      key: 'description',
      type: 'input',
      span: 20,
      labelWidth: '120px',
      inputType: 'textarea',
    },
    {
      label: '图标',
      key: 'icons',
      type: 'selectIcon',
      span: 20,
      labelWidth: '120px',
      hidden: isIconsHidden,
    },
    {
      label: '前端url',
      key: 'path',
      type: 'input',
      span: 20,
      labelWidth: '120px',
      hidden: isPathAndCmpPathHidden,
      disabled: isDisabled,
      placeholder: placeholder,
    },
    {
      label: '组件路径',
      key: 'cmpPath',
      type: 'input',
      span: 20,
      labelWidth: '120px',
      hidden: isCmpPathHidden,
      disabled: isDisabled,
      placeholder: '前端代码组件相对地址，如：/group/menu/index',
    },
    {
      label: '排序值',
      key: 'sort',
      type: 'inputNumber',
      options: [],
      span: 20,
      labelWidth: '120px',
      min: 0,
      max: 100,
      value: 0,
    },
    {
      label: '开启标志',
      key: 'state',
      type: 'radio',
      options: [
        { label: '启用', value: 1 },
        { label: '禁用', value: 0 },
      ],
      span: 20,
      labelWidth: '120px',
      value: 1,
      disabled: isSystemDisabled,
    },
  ]);

  const optionList = computed(() => MyOptionList.filter((e) => !e.hidden));
  return {
    optionList,
  };
}
