import { toRefs, reactive, onMounted, computed, nextTick } from 'vue';
import type { ComputedRef, Ref } from 'vue';
import type { FormItemRule } from 'element-plus/es/tokens/form.d';
import type { FormOption, SearchOption, FormSelectOption, TreeProps } from '@/types/tools';
import { getRecipient, getDictionaryTree } from '@/api/permission/roleManagement';
import { getOrgAll, getDistrict } from '@/api/permission/organization';
import { getPostList } from '@/api/permission/postManagement';
import { getDictionaryList } from '@/api/systemManagement/dictionaryManagement';

type MyFormOption = Omit<FormOption, 'disabled'> & {
  hidden?: boolean | ComputedRef<boolean>;
  disabled?: boolean | ComputedRef<boolean>;
};

interface MyFormData {
  type: number;
  roleType: number;
}

export default function useOptions(
  currentData: Ref<MyFormData | null>,
  formRef: any,
  roleTypeOptions: FormSelectOption[],
  roleTypeDisabled: boolean,
  formDisabled: boolean,
  handleSetFormValue: any,
): any {
  const optionsState = reactive<{
    relatedOptions: FormSelectOption[];
    orgTreeOptions: FormSelectOption[];
    districtTreeOptions: FormSelectOption[];
    postListOptions: FormSelectOption[];
    recipientOptions: FormSelectOption[];
    dataCodeOptions: FormSelectOption[];
    dataCopyTypeOptions: any[];
    relatedProps: TreeProps;
    orgProps: TreeProps;
    districtProps: TreeProps;
    recipientProps: TreeProps;
  }>({
    relatedOptions: [],
    orgTreeOptions: [],
    districtTreeOptions: [],
    relatedProps: {},
    orgProps: {
      label: 'orgName',
      children: 'children',
    },
    districtProps: {
      label: 'name',
      children: 'children',
    },
    postListOptions: [],
    recipientOptions: [],
    recipientProps: {
      label: 'title',
      children: 'children',
    },
    dataCodeOptions: [],
    dataCopyTypeOptions: [],
  });
  const { relatedOptions, relatedProps, postListOptions, dataCopyTypeOptions, dataCodeOptions } =
    toRefs(optionsState);

  // 获取指定组织下拉选择树
  const getOrgAllTree = async () => {
    getOrgAll().then((res) => {
      optionsState.orgTreeOptions = res.data;
    });
  };
  // 获取行政区域下拉值
  const getDistrictTree = async () => {
    getDistrict().then((res) => {
      optionsState.districtTreeOptions = res.data;
    });
  };

  // 获取指定人下拉值
  const getRecipientTree = async () => {
    getRecipient({ filterNullOrg: true }).then((res) => {
      optionsState.recipientOptions = res.data;
    });
  };

  // 获取指定岗位下拉值
  const getPostListOptions = async () => {
    getPostList().then((res) => {
      optionsState.postListOptions = res.data;
    });
  };

  // 获取数据字典-类别下拉值
  const getDictionaryListOptions = async () => {
    getDictionaryList().then((res) => {
      optionsState.dataCopyTypeOptions = [
        {
          id: 'businessList',
          typeName: '业务类型',
          typeValue: '业务类型',
          children: res.data.businessList,
        },
        {
          id: 'systemList',
          typeName: '系统类型',
          typeValue: '系统类型',
          children: res.data.systemList,
        },
      ];
    });
  };

  // 获取数据字典-类别-具体字典项
  const getDictionaryTreeOptions = async (code) => {
    getDictionaryTree(code).then((res) => {
      optionsState.dataCodeOptions = res.data[code];
    });
  };

  // 清空数据作用域相关字段值
  const clearTypeOfForm = () => {
    formRef.value.form.relatedIds = [];
    formRef.value.form.relatedId = '';
    optionsState.relatedOptions = [];
  };

  // 判断是否显示数据作用域（只有类型为角色时才显示）
  const isTypeHidden = computed(
    () => ![0].includes(currentData.value ? currentData.value.roleType : 0),
  );
  // 判断是否显示指定组织下拉控件
  const isRelatedHidden = computed(
    () => ![1, 3, 7].includes(currentData.value ? currentData.value.type : 0),
  );

  // 判断是否显示指定岗位下拉控件
  const isPostHidden = computed(
    () => ![2].includes(currentData.value ? currentData.value.type : 0),
  );

  // 判断是否显示数据字典下拉控件
  const isDataCodeTypeHidden = computed(
    () => ![8].includes(currentData.value ? currentData.value.type : 0),
  );

  const isDataCodeHidden = computed(() => {
    if (formRef.value?.form?.relatedId) {
      getDictionaryTreeOptions(formRef.value.form.relatedId);
      return false;
    } else return true;
  });

  // 分类更改事件
  const roleTypeChangeEvent = computed(() => (val) => {
    if (val) {
      currentData.value!.type = 0;
      formRef.value.form.type = 0;
      clearTypeOfForm();
    }
    currentData.value!.roleType = val;
  });

  // 数据作用域类型更改事件
  const typeChangeEvent = (val) => {
    clearTypeOfForm();
    switch (val) {
      case 1:
        optionsState.relatedOptions = optionsState.orgTreeOptions;
        optionsState.relatedProps = optionsState.orgProps;
        break;
      case 3:
        optionsState.relatedOptions = optionsState.recipientOptions;
        optionsState.relatedProps = optionsState.recipientProps;
        break;
      case 7:
        optionsState.relatedOptions = optionsState.districtTreeOptions;
        optionsState.relatedProps = optionsState.districtProps;
        break;
      default:
        break;
    }
    handleSetFormValue(formRef.value.form);
    nextTick(() => {
      currentData.value!.type = val;
    });
  };
  // 初始化查询下拉
  const typeFirstChangeEvent = (val) => {
    switch (val) {
      case 1:
        optionsState.relatedOptions = optionsState.orgTreeOptions;
        optionsState.relatedProps = optionsState.orgProps;
        break;
      case 3:
        optionsState.relatedOptions = optionsState.recipientOptions;
        optionsState.relatedProps = optionsState.recipientProps;
        break;
      case 7:
        optionsState.relatedOptions = optionsState.districtTreeOptions;
        optionsState.relatedProps = optionsState.districtProps;
        break;
      default:
        break;
    }
  };
  const state = reactive<{
    searchOptions: SearchOption[];
    columns: any[];
    formRules: Partial<Record<string, FormItemRule | FormItemRule[]>>;
  }>({
    searchOptions: [
      {
        label: '角色分组',
        key: 'classify',
        type: 'select',
        options: [
          {
            value: '',
            label: '所有',
          },
          {
            value: '0',
            label: '业务',
          },
          {
            value: '1',
            label: '系统',
          },
        ],
      },
      {
        label: '',
        key: 'keyword',
        type: 'text',
        placeholder: '角色名称/角色编码/角色描述',
        labelWidth: '0px',
        span: 8,
      },
    ],
    columns: [
      {
        prop: 'roleName',
        label: '角色名称',
        minWidth: 100,
        align: 'center',
      },
      {
        prop: 'roleCode',
        label: '角色编码',
        minWidth: 100,
        align: 'center',
      },
      {
        prop: 'sort',
        label: '排序值',
        minWidth: 100,
        align: 'center',
      },
      {
        prop: 'roleDesc',
        label: '角色描述',
        minWidth: 100,
        align: 'center',
      },
      {
        prop: 'state',
        label: '状态',
        minWidth: 100,
        align: 'center',
        slot: 'state',
      },
    ],
    formRules: {
      positionName: [
        {
          required: true,
          message: '请选择分组',
          trigger: 'change',
        },
      ],
      roleName: [
        {
          required: true,
          message: '请输入角色名称',
          trigger: 'blur',
        },
      ],
      roleCode: [
        {
          required: true,
          message: '请输入角色编码',
          trigger: 'blur',
        },
        {
          message: '请输入正确的编码格式数字、英文',
          trigger: 'blur',
          validator: (rule, value) => /^[a-zA-Z0-9]+$/.test(value),
        },
      ],
      sort: [
        {
          required: true,
          message: '请输入排序值',
          trigger: 'blur',
        },
      ],
    },
  });

  const formOptionList: MyFormOption[] = reactive([
    {
      label: '角色名称',
      key: 'roleName',
      type: 'input',
      span: 12,
      labelWidth: '120px',
      disabled: formDisabled,
    },
    {
      label: '角色编码',
      key: 'roleCode',
      type: 'input',
      span: 12,
      labelWidth: '120px',
      disabled: formDisabled,
    },
    {
      label: '类型',
      key: 'roleType',
      type: 'radio',
      span: 12,
      labelWidth: '120px',
      disabled: roleTypeDisabled,
      changeEvent: roleTypeChangeEvent,
      value: 0,
      options: [
        {
          label: '角色',
          value: 0,
        },
        {
          label: '分组',
          value: 1,
        },
      ],
    },
    {
      label: '父节点',
      key: 'parentId',
      type: 'treeSelect',
      span: 12,
      labelWidth: '120px',
      disabled: formDisabled,
      checkStrictly: true,
      options: roleTypeOptions as any,
      dataKey: 'id',
      props: {
        label: 'name',
        children: 'children',
      },
    },
    {
      label: '排序值',
      key: 'sort',
      type: 'inputNumber',
      disabled: formDisabled,
      span: 12,
      labelWidth: '120px',
      value: 1,
      min: 1,
    },
    {
      label: '状态',
      key: 'state',
      type: 'switch',
      disabled: formDisabled,
      span: 12,
      labelWidth: '120px',
      value: 1,
    },
    {
      label: '数据作用域',
      key: 'type',
      type: 'select',
      span: 12,
      disabled: formDisabled,
      labelWidth: '120px',
      hidden: isTypeHidden,
      changeEvent: typeChangeEvent,
      options: [
        { label: '所有数据', value: 0 },
        { label: '指定组织', value: 1 },
        { label: '指定岗位', value: 2 },
        { label: '指定人', value: 3 },
        { label: '创建者组织', value: 4 },
        { label: '创建者岗位', value: 5 },
        { label: '创建人', value: 6 },
        { label: '行政区域', value: 7 },
        { label: '数据字典', value: 8 },
      ],
    },
    {
      label: '',
      key: 'relatedIds',
      type: 'treeSelect',
      span: 6,
      labelWidth: '5px',
      disabled: formDisabled,
      showCheckbox: true,
      hidden: isRelatedHidden,
      options: relatedOptions as any,
      dataKey: 'id',
      props: relatedProps as any,
      multiple: true,
      collapseTags: true,
      collapseTagsTooltip: true,
    }, // 指定组织、行政区域和指定人
    {
      label: '',
      key: 'relatedIds',
      type: 'select',
      span: 6,
      disabled: formDisabled,
      labelWidth: '5px',
      multiple: true,
      hidden: isPostHidden,
      options: postListOptions as any,
      optionValue: 'id',
      optionLabel: 'positionName',
    }, // 指定岗位
    {
      label: '类别',
      key: 'relatedId',
      type: 'treeSelect',
      disabled: formDisabled,
      span: 6,
      labelWidth: '60px',
      hidden: isDataCodeTypeHidden,
      options: dataCopyTypeOptions as any,
      dataKey: 'typeValue',
      props: {
        label: 'typeName',
        children: 'children',
      },
      changeEvent: () => {
        formRef.value.form.relatedIds = [];
        handleSetFormValue(formRef.value.form);
      },
      filterMethod: (data, value) => (value.children ? false : value.typeName.includes(data)),
    }, // 数据字典-类别
    {
      label: '',
      key: 'relatedIds',
      type: 'treeSelect',
      span: 6,
      labelWidth: '5px',
      disabled: formDisabled,
      showCheckbox: true,
      hidden: isDataCodeHidden,
      options: dataCodeOptions as any,
      dataKey: 'id',
      props: {
        label: 'dictValue',
        children: 'children',
      },
      multiple: true,
      collapseTags: true,
      collapseTagsTooltip: true,
    }, // 数据字典
    {
      label: '角色描述',
      key: 'roleDesc',
      type: 'input',
      disabled: formDisabled,
      span: 24,
      labelWidth: '120px',
      inputType: 'textarea',
      rows: '2',
    },
  ]);

  // 请求下拉列表选项
  const initOptions = async () => {
    getOrgAllTree();
    getDistrictTree();
    getPostListOptions();
    getRecipientTree();
    getDictionaryListOptions();
  };

  const isComputedHiddenList = computed(() => formOptionList.filter((e) => !e.hidden));

  onMounted(async () => {
    initOptions();
  });

  return {
    ...toRefs(state),
    isComputedHiddenList,
    typeFirstChangeEvent,
  };
}
