import { toRefs, reactive, nextTick, computed } from 'vue';
import type { FormItemRule } from 'element-plus/es/tokens/form.d';

import { ElMessageBox } from 'element-plus';
import {
  addDictionaryType,
  editDictionaryType,
  deleteDictionaryType,
  isDictHasChildren,
} from '@/api/systemManagement/dictionaryManagement';
import type { FormOption } from '@/types/tools';

export default function useAddType(): any {
  const state = reactive<{
    addTypeDialogVisible: boolean;
    addTypeFormRef: any;
    addTypeFormValue: any;
    saveTypeLoading: boolean;
    addTypeDialogType: string;
    addTypeEditData: any;
  }>({
    addTypeDialogVisible: false,
    addTypeFormRef: null,
    addTypeFormValue: null,
    saveTypeLoading: false,
    addTypeDialogType: '',
    addTypeEditData: null,
  });

  // 判断表单是否可编辑
  const isClassifyDisable = computed(() => state.addTypeDialogType === 'edit');
  const isTypeNameDisable = computed(
    () => state.addTypeDialogType === 'edit' && state.addTypeEditData.classify === 1,
  );
  const isTypeValueDisable = computed(() => state.addTypeDialogType === 'edit');

  const addTypeFormOptionList: FormOption[] = reactive([
    {
      label: '分组',
      key: 'classify',
      type: 'select',
      span: 12,
      labelWidth: '120px',
      options: [
        {
          label: '系统',
          value: 1,
        },
        {
          label: '业务',
          value: 0,
        },
      ],
      disabled: isClassifyDisable,
    },
    {
      label: '类型名称',
      key: 'typeName',
      type: 'input',
      span: 12,
      labelWidth: '120px',
      disabled: isTypeNameDisable,
    },
    {
      label: '类型编码',
      key: 'typeValue',
      type: 'input',
      span: 12,
      labelWidth: '120px',
      disabled: isTypeValueDisable,
    },
    {
      label: '排序值',
      key: 'sort',
      type: 'inputNumber',
      span: 12,
      labelWidth: '120px',
      value: 1,
      min: 1,
    },
  ]);

  const addTypeFormRules: Partial<Record<string, FormItemRule | FormItemRule[]>> = {
    classify: [
      {
        required: true,
        message: '请选择分组',
        trigger: 'change',
      },
    ],
    typeName: [
      {
        required: true,
        message: '请输入类型名称',
        trigger: 'blur',
      },
    ],
    typeValue: [
      {
        required: true,
        message: '请输入类型编码',
        trigger: 'blur',
      },
    ],
    sort: [
      {
        required: true,
        message: '请输入排序值',
        trigger: 'blur',
      },
    ],
  };

  // 打开弹框
  const handleOpenTypeDialog = (type: string, data: any = null) => {
    state.addTypeDialogVisible = true;
    state.addTypeDialogType = type;
    if (type === 'edit') {
      state.addTypeEditData = data;
      state.addTypeFormValue = data;
    }
    if (type === 'add' && !!data) {
      state.addTypeFormValue = data;
    }
  };

  // 弹框标题
  const addTypeDialogTitle = computed(() => {
    if (state.addTypeDialogType === 'add') {
      return '字典类型新增';
    } else if (state.addTypeDialogType === 'edit') {
      return '字典类型信息';
    } else if (state.addTypeDialogType === 'addChild') {
      return '字典子项新增';
    } else {
      return '';
    }
  });

  // 取消新增字典类型
  const handleCancelAddTypeDialog = () => {
    state.addTypeFormValue = null;
    state.addTypeDialogVisible = false;
    nextTick(() => {
      state.addTypeFormRef.restoreForm();
    });
  };

  // 保存字典
  const handleSaveType = (initData: () => void) => {
    state.addTypeFormRef.getValue().then((res: any) => {
      if (res) {
        state.saveTypeLoading = true;
        if (state.addTypeDialogType === 'edit') {
          res.id = state.addTypeEditData.id;
        }
        const _api = state.addTypeDialogType === 'add' ? addDictionaryType : editDictionaryType;
        _api(res)
          .then(() => {
            state.addTypeDialogVisible = false;
            initData();
          })
          .finally(() => {
            state.saveTypeLoading = false;
          });
      }
    });
  };

  // 删除左侧列表的
  const handleDeleteType = (row: any, initData: () => void) => {
    ElMessageBox.confirm('是否删除该字典类型？', {
      confirmButtonText: '确定',
      cancelButtonText: '取消',
      type: 'warning',
    })
      .then(async () => {
        const { data: canNotDelete } = await isDictHasChildren([row.id]);
        if (canNotDelete) {
          ElMessageBox.confirm('该字典类型包含多个配置项，删除后将无法恢复，是否继续？', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning',
          })
            .then(async () => {
              try {
                await deleteDictionaryType([{ id: row.id, positionName: row.positionName }]);
                await initData();
              } catch (err) {}
            })
            .catch(() => {});
        } else {
          try {
            await deleteDictionaryType([{ id: row.id, positionName: row.positionName }]);
            await initData();
          } catch (err) {}
        }
      })
      .catch(() => {});
  };

  return {
    ...toRefs(state),
    handleCancelAddTypeDialog,
    addTypeFormOptionList,
    addTypeFormRules,
    handleSaveType,
    addTypeDialogTitle,
    handleOpenTypeDialog,
    handleDeleteType,
  };
}
