import { toRefs, reactive, computed, nextTick } from 'vue';
import type { FormItemRule } from 'element-plus/es/tokens/form.d';
import { ElMessageBox, ElMessage } from 'element-plus';
import type { FormOption, SearchOption, FormSelectOption } from '@/types/tools';
import { createToDownloadFile } from '@/utils/tools';
import {
  disableOrEnable,
  exportAll,
  exportDict,
  getDictionaryDetail,
  editDictionaryData,
  addDictionaryData,
  deleteDictionaryData,
  getParentNodesOptions,
} from '@/api/systemManagement/dictionaryManagement';

interface MyFormOption extends FormOption {
  isShow?: boolean;
}

const optionsData = {
  fileType: [
    {
      label: '文件夹',
      value: 1,
    },
    {
      label: '文件',
      value: 0,
    },
  ],
  state: [
    {
      value: '',
      label: '所有',
    },
    {
      value: 1,
      label: '启用',
    },
    {
      value: 0,
      label: '禁用',
    },
  ],
};
export default function useOperateConfig(activeItemData: any): any {
  const state = reactive<{
    exportConfigLoading: boolean;
    addConfigDialogVisible: boolean;
    addConfigDialogType: string;
    addConfigFormValue: any;
    addConfigFormRef: any;
    addConfigFormShowOptionList: FormOption[];
    configFormDisabled: boolean;
    saveConfigLoading: boolean;
    addConfigEditData: any;
    searchOptions: SearchOption[];
    columns: any[];
    pidNameOptions: FormSelectOption[];
  }>({
    exportConfigLoading: false,
    addConfigDialogVisible: false,
    addConfigDialogType: '',
    addConfigFormValue: null,
    addConfigFormRef: null,
    saveConfigLoading: false,
    addConfigEditData: null,
    addConfigFormShowOptionList: [],
    configFormDisabled: false,
    searchOptions: [
      {
        label: '状态',
        key: 'state',
        type: 'select',
        options: optionsData.state,
      },
      {
        label: '',
        key: 'keyword',
        type: 'text',
        placeholder: '配置项名称/编码',
        labelWidth: '0px',
      },
    ],
    columns: [
      {
        prop: 'dictValue',
        label: '字典名称',
        minWidth: 100,
      },
      {
        prop: 'dictKey',
        label: '字典编码',
        minWidth: 100,
      },
      {
        prop: 'label',
        label: '标签',
        minWidth: 100,
      },
      {
        prop: 'state',
        label: '状态',
        minWidth: 100,
        align: 'center',
        slot: 'state',
      },
      {
        prop: 'sort',
        label: '排序值',
        minWidth: 100,
      },
      {
        prop: 'remarks',
        label: '描述',
        minWidth: 100,
      },
    ],
    pidNameOptions: [],
  });

  const { pidNameOptions } = toRefs(state);

  // 判断表单项是否显示
  // const isShowPidName = computed(
  //   () =>
  //     state.addConfigDialogType === 'addChild' ||
  //     (state.addConfigDialogType === 'edit' && state.addConfigEditData.pid !== '-1'),
  // );

  // 判断表单项是否可编辑
  const isDisabled = computed(() => state.addConfigDialogType === 'edit');

  const addConfigFormOptionList: MyFormOption[] = reactive([
    {
      label: '字典名称',
      key: 'dictValue',
      type: 'input',
      span: 12,
      labelWidth: '120px',
      isShow: true,
      maxlength: 8,
    },
    {
      label: '字典编码',
      key: 'dictKey',
      type: 'input',
      span: 12,
      labelWidth: '120px',
      isShow: true,
    },
    {
      label: '字典类型',
      key: 'typeName',
      type: 'input',
      span: 12,
      labelWidth: '120px',
      disabled: true,
      isShow: true,
    },
    {
      label: '父节点',
      key: 'pid',
      type: 'treeSelect',
      span: 12,
      labelWidth: '120px',
      checkStrictly: true,
      options: pidNameOptions as any,
      isShow: true,
      dataKey: 'id',
      props: {
        label: 'dictValue',
        children: 'children',
      },
    },
    {
      label: '配置类型',
      key: 'isCatalogue',
      type: 'radio',
      span: 12,
      labelWidth: '120px',
      options: [
        {
          label: '虚拟分组',
          value: 1,
        },
        {
          label: '配置项',
          value: 0,
        },
      ],
      isShow: true,
      disabled: isDisabled,
    },
    {
      label: '标签',
      key: 'label',
      type: 'input',
      span: 12,
      labelWidth: '120px',
      isShow: true,
    },
    {
      label: '排序值',
      key: 'sort',
      type: 'inputNumber',
      span: 12,
      labelWidth: '120px',
      value: 1,
      min: 1,
      isShow: true,
    },
    {
      label: '状态',
      key: 'state',
      type: 'switch',
      span: 12,
      labelWidth: '120px',
      value: 1,
      isShow: true,
    },
    {
      label: '描述',
      key: 'remarks',
      type: 'input',
      span: 24,
      labelWidth: '120px',
      inputType: 'textarea',
      rows: '2',
      isShow: true,
    },
  ]);
  const addConfigFormRules: Partial<Record<string, FormItemRule | FormItemRule[]>> = {
    isCatalogue: [
      {
        required: true,
        message: '请选择配置类型',
        trigger: 'change',
      },
    ],
    dictKey: [
      {
        required: true,
        message: '请输入字典编码',
        trigger: 'blur',
      },
    ],
    dictValue: [
      {
        required: true,
        message: '请输入字典名称',
        trigger: 'blur',
      },
    ],
    state: [
      {
        required: true,
        message: '请选择状态',
        trigger: 'change',
      },
    ],
    sort: [
      {
        required: true,
        message: '请输入排序值',
        trigger: 'blur',
      },
    ],
  };

  // 获取父节点下拉值选项
  const getParentNodes = (code) => {
    getParentNodesOptions({ dictTypeCode: code }).then((res) => {
      res.data.push({
        id: '-1',
        dictValue: '根节点',
        children: [],
      });
      state.pidNameOptions = res.data;
    });
  };

  // 修改状态
  const handleChangeState = (checked: boolean, record: any, initData: () => void) => {
    if (!record.id) return;
    const param = {
      id: record.id,
      state: Number(checked),
    };
    disableOrEnable(param).finally(() => {
      initData();
    });
  };

  // 导出所有字典配置
  const exportAllDictionaryConfig = () => {
    state.exportConfigLoading = true;
    exportAll()
      .then((_res) => {
        const { data } = _res;
        createToDownloadFile(data.data, data.fileName);
      })
      .finally(() => {
        state.exportConfigLoading = false;
      });
  };

  // 导出当前字典配置
  const exportDictionaryConfig = (id: string) => {
    state.exportConfigLoading = true;
    exportDict({ dictTypeId: id })
      .then((_res) => {
        const { data } = _res;
        createToDownloadFile(data.data, data.fileName);
      })
      .finally(() => {
        state.exportConfigLoading = false;
      });
  };

  // 打开弹框
  const handleOpenConfigDialog = (type: string, data: any = null) => {
    state.addConfigDialogVisible = true;
    state.addConfigDialogType = type;
    state.addConfigEditData = data;
    const dictTypeCode = `${activeItemData.value.typeName}（${activeItemData.value.typeValue}）`;
    getParentNodes(activeItemData.value.typeValue);
    switch (type) {
      case 'add':
        state.configFormDisabled = false;
        state.addConfigFormValue = {
          typeName: dictTypeCode,
          isCatalogue: 0,
        };
        state.addConfigFormShowOptionList = addConfigFormOptionList.filter((e) => e.isShow);
        break;
      case 'edit':
        state.configFormDisabled = true;
        getDictionaryDetail(data.id).then((_res) => {
          const { data: dictData } = _res;
          dictData.typeName = dictTypeCode;
          state.addConfigEditData = dictData;
          state.addConfigFormValue = dictData;
          state.addConfigFormShowOptionList = addConfigFormOptionList.filter((e) => e.isShow);
        });
        break;
      case 'addChild':
        state.configFormDisabled = false;
        state.addConfigFormValue = {
          typeName: dictTypeCode,
          isCatalogue: 0,
          pid: data.id,
          // pidName: `${data.dictValue}(${data.dictKey})`,
        };
        state.addConfigFormShowOptionList = addConfigFormOptionList.filter((e) => e.isShow);
        break;
      default:
        break;
    }
  };

  // 弹框标题
  const addConfigDialogTitle = computed(() => {
    if (state.addConfigDialogType === 'add') {
      return '字典类型新增';
    } else if (state.addConfigDialogType === 'edit') {
      return '字典类型信息';
    } else if (state.addConfigDialogType === 'addChild') {
      return '字典子项新增';
    } else {
      return '';
    }
  });

  // 取消新增字典配置
  const handleCancelAddConfigDialog = () => {
    state.addConfigFormValue = null;
    state.addConfigDialogVisible = false;
    nextTick(() => {
      state.addConfigFormRef.restoreForm();
    });
  };

  // 保存字典配置
  const handleSaveConfig = (initData: () => void) => {
    if (state.addConfigDialogType === 'edit' && state.configFormDisabled) {
      state.configFormDisabled = false;
      return;
    }
    state.addConfigFormRef.getValue().then((res: any) => {
      if (res) {
        state.saveConfigLoading = true;
        res.typeValue = activeItemData.value.typeValue;
        res.classify = activeItemData.value.classify;
        if (state.addConfigDialogType === 'add') {
        } else if (state.addConfigDialogType === 'edit') {
          // res.pid = state.addConfigEditData.pid;
          res.id = state.addConfigEditData.id;
        }
        const _api =
          (['add', 'addChild'].includes(state.addConfigDialogType) && addDictionaryData) ||
          editDictionaryData;
        _api({
          ...res,
          dictTypeId: activeItemData.value.id,
        })
          .then(() => {
            state.addConfigDialogVisible = false;
            initData();
          })
          .finally(() => {
            state.saveConfigLoading = false;
          });
      }
    });
  };

  // （批量）删除字典配置数据
  const handleDeleteConfig = async (record, initData: () => void) => {
    // 判断是否有数据
    if (!record.length) {
      ElMessage.warning('请选择数据！');
      return;
    }
    // 弹框是否删除
    try {
      await ElMessageBox.confirm('是否删除', {
        confirmButtonText: '确定',
        cancelButtonText: '取消',
        type: 'warning',
      });
    } catch (err) {
      return;
    }
    // 判断选中的数据是否有启用的情况
    const hasEnableArr = record.filter((item: any) => item.state === 1);
    if (hasEnableArr.length > 0) {
      try {
        await ElMessageBox.confirm('选中的数据中存在已启用的配置，是否继续？', {
          confirmButtonText: '确定',
          cancelButtonText: '取消',
          type: 'warning',
        });
      } catch (err) {
        return;
      }
    }
    // 删除逻辑
    const params = record.map((record) => ({ id: record.id, dictValue: record.dictValue }));
    try {
      await deleteDictionaryData(params);
      await initData();
    } catch (err) {}
  };
  return {
    ...toRefs(state),
    handleChangeState,
    exportAllDictionaryConfig,
    exportDictionaryConfig,
    addConfigDialogTitle,
    handleCancelAddConfigDialog,
    addConfigFormRules,
    handleSaveConfig,
    handleOpenConfigDialog,
    handleDeleteConfig,
  };
}
