import { toRefs, reactive, onMounted, onBeforeUnmount } from 'vue';
import * as echarts from 'echarts/core';
import type { GridComponentOption, LegendComponentOption } from 'echarts/components';
import { GridComponent, LegendComponent } from 'echarts/components';
import type { BarSeriesOption } from 'echarts/charts';
import { BarChart } from 'echarts/charts';
import { CanvasRenderer } from 'echarts/renderers';
import { throttle } from '@/utils/tools';

echarts.use([GridComponent, LegendComponent, BarChart, CanvasRenderer]);

type EChartsOption = echarts.ComposeOption<
  GridComponentOption | LegendComponentOption | BarSeriesOption
>;
interface HistogramChartData {
  name: string;
  value: number;
  percent: number;
  color: string;
}
export default function useDrawHistogramChart(): any {
  const state = reactive<{
    histogramChart: echarts.ECharts | null;
    histogramChartDOMId: string;
  }>({
    histogramChart: null,
    histogramChartDOMId: `echart-${Date.now()}-${Math.random()}`,
  });
  const handleResizeChart = throttle(() => {
    state.histogramChart?.resize();
  }, 100);
  onMounted(() => {
    state.histogramChart = echarts.init(document.getElementById(state.histogramChartDOMId)!);
    window.addEventListener('resize', handleResizeChart);
  });
  onBeforeUnmount(() => {
    window.removeEventListener('resize', handleResizeChart);
  });

  const handleDrawHistogramChart = (data: HistogramChartData[] = []) => {
    const option: EChartsOption = {
      xAxis: {
        type: 'category',
        data: data.map((e) => e.name),
      },
      yAxis: {
        type: 'value',
        name: '数量（条）',
      },
      legend: {
        orient: 'vertical',
        left: 'left',
      },
      tooltip: {
        trigger: 'item',
      },
      series: [
        {
          data: data.map((i) => i.value),
          barWidth: '20%',
          color: data.map((i) => i.color),
          type: 'bar',
          colorBy: 'data',
          label: {
            show: true,
            position: 'top',
          },
        },
      ],
    };
    const unwarp = (obj) => obj && (obj.__v_raw || obj.valueOf() || obj);
    option && unwarp(state.histogramChart)!.setOption(option);
  };
  return {
    ...toRefs(state),
    handleDrawHistogramChart,
  };
}
