import BigNumber from 'bignumber.js';
import { toRefs, reactive, onMounted, onBeforeUnmount } from 'vue';
import * as echarts from 'echarts/core';
import type {
  TitleComponentOption,
  TooltipComponentOption,
  LegendComponentOption,
} from 'echarts/components';
import { TitleComponent, TooltipComponent, LegendComponent } from 'echarts/components';
import type { PieSeriesOption } from 'echarts/charts';
import { PieChart } from 'echarts/charts';
import { LabelLayout } from 'echarts/features';
import { CanvasRenderer } from 'echarts/renderers';
import { throttle } from '@/utils/tools';

echarts.use([
  TitleComponent,
  TooltipComponent,
  LegendComponent,
  PieChart,
  CanvasRenderer,
  LabelLayout,
]);
type EChartsOption = echarts.ComposeOption<
  TitleComponentOption | TooltipComponentOption | LegendComponentOption | PieSeriesOption
>;

export default function useDrawPieChart(): any {
  const state = reactive<{
    pieChart: echarts.ECharts | null;
    pieChartId: string;
  }>({
    pieChart: null,
    pieChartId: `echart-${Date.now()}-${Math.random()}`,
  });
  const handleResizeChart = throttle(() => {
    state.pieChart?.resize();
  }, 100);

  onMounted(() => {
    state.pieChart = echarts.init(document.getElementById(state.pieChartId)!);
    window.addEventListener('resize', handleResizeChart);
  });
  onBeforeUnmount(() => {
    window.removeEventListener('resize', handleResizeChart);
  });

  const handleDrawPieChart = (data: any = []) => {
    const option: EChartsOption = {
      tooltip: {
        trigger: 'item',
      },
      legend: {
        orient: 'vertical',
        left: 'left',
      },
      label: {
        formatter: (params) => `${params.data.name}: ${BigNumber(params.data.percent).times(100)}%`,
      },
      series: [
        {
          type: 'pie',
          radius: '60%',
          data: data,
          color: data.map((i) => i.color),
          emphasis: {
            itemStyle: {
              shadowBlur: 10,
              shadowOffsetX: 0,
              shadowColor: 'rgba(0, 0, 0, 0.5)',
            },
          },
        },
      ],
    };

    const unwarp = (obj) => obj && (obj.__v_raw || obj.valueOf() || obj);
    option && unwarp(state.pieChart)!.setOption(option);
  };
  return {
    ...toRefs(state),
    handleDrawPieChart,
  };
}
