import { toRefs, reactive, onMounted, onBeforeUnmount, nextTick } from 'vue';
import * as echarts from 'echarts/core';
import type {
  TooltipComponentOption,
  GridComponentOption,
  LegendComponentOption,
} from 'echarts/components';
import { TooltipComponent, GridComponent, LegendComponent } from 'echarts/components';
import type { LineSeriesOption } from 'echarts/charts';
import { LineChart } from 'echarts/charts';
import { UniversalTransition } from 'echarts/features';
import { CanvasRenderer } from 'echarts/renderers';
import { throttle } from '@/utils/tools';

echarts.use([
  TooltipComponent,
  GridComponent,
  LegendComponent,
  LineChart,
  CanvasRenderer,
  UniversalTransition,
]);

type EChartsOption = echarts.ComposeOption<
  TooltipComponentOption | GridComponentOption | LegendComponentOption | LineSeriesOption
>;

export default function useLineChar(): any {
  const state = reactive<{
    lineChart: echarts.ECharts | null;
    lineChartDOMId: string;
  }>({
    lineChart: null,
    lineChartDOMId: `echart-${Date.now()}-${Math.random()}`,
  });
  const handleResizeChart = throttle(() => {
    state.lineChart?.resize();
  }, 100);
  onMounted(() => {
    state.lineChart = echarts.init(document.getElementById(state.lineChartDOMId)!);
    window.addEventListener('resize', handleResizeChart);
  });

  onBeforeUnmount(() => {
    window.removeEventListener('resize', handleResizeChart);
  });

  const handleDrawLineChart = async (data: any) => {
    const option: EChartsOption = {
      tooltip: {
        trigger: 'axis',
      },
      legend: {
        data: data.series.map((i) => i.name),
      },
      grid: {
        left: '3%',
        right: '4%',
        bottom: '3%',
        containLabel: true,
      },
      xAxis: {
        type: 'category',
        boundaryGap: false,
        data: data.axis,
      },
      yAxis: {
        type: 'value',
      },
      series: data.series as LineSeriesOption,
    };
    // 配置tooltip不显示问题 https://github.com/apache/echarts/issues/15413
    const unwarp = (obj) => obj && (obj.__v_raw || obj.valueOf() || obj);
    unwarp(state.lineChart).clear();
    await nextTick();
    option && unwarp(state.lineChart).setOption(option);
  };
  return {
    ...toRefs(state),
    handleDrawLineChart,
  };
}
